import logging;
from datetime import datetime;
import AlmaxUtils.Time as AUT;
import os;

class LoggerService:
    def __init__(self, programNameLogger: str, details: str = '', genericLogFileName: str = "__0.ExecutionLog.log", addTimeFolder: bool = True, logFormat: str = '%(asctime)s - %(message)s', absoluteLogPath: str = ''):
        self.__LogPath = f"{absoluteLogPath}{programNameLogger}{details}";
       
        if addTimeFolder:
            self.__LogPath += "/" + datetime.now().strftime("%Y%m%d_%H%M%S");
        if not (os.path.exists(self.__LogPath)):
            os.makedirs(self.__LogPath);
       
        self.__Istance = logging.getLogger(programNameLogger + details);
        self.file_handler = logging.FileHandler(filename=f"{self.__LogPath}/{genericLogFileName}", mode='a');
        self.formatter = logging.Formatter(logFormat);
        self.file_handler.setFormatter(self.formatter);
        self.__Istance.addHandler(self.file_handler);
        self.__Istance.setLevel(logging.INFO);
       
   
        self.__CustomTimer = datetime.now();
        self.__Timer = datetime.now();
   
    def AddLog(self, message):
        self.__Istance.info(message);

    def StartTiming(self, message):
        self.__Istance.info(message);
        self.__CustomTimer = datetime.now();

    def EndTiming(self):
        self.__Istance.info(f"\tDuration: {AUT.TimeToString(AUT.CalculateTimePassed(self.__CustomTimer))}");
   
    def StartGenericTiming(self):
        self.__Timer = datetime.now();

    def ForceEnd(self):
        self.__Istance.info(f"Total Duration: {AUT.TimeToString(AUT.CalculateTimePassed(self.__Timer))}");
        self.__Istance.info(f"FORCED END\n\n");

    def Start(self):
        self.__Istance.info(f"START");

    def End(self):
        self.__Istance.info(f"Total Duration: {AUT.TimeToString(AUT.CalculateTimePassed(self.__Timer))}");
        self.__Istance.info(f"END\n\n");

    def LogPath(self) -> str:
        return self.__LogPath;


import AlmaxUtils.Time as TimeLib
import os
import pypdf

from reportlab.lib.pagesizes import A4
from reportlab.lib import colors
from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
from reportlab.platypus import SimpleDocTemplate, Paragraph, Spacer, Table, TableStyle


def GeneratePdf(client_info, orders, totalObj, finalText: list, documentType: str) -> str:
    now = TimeLib.now
    now_month = now.month if now.month > 9 else f"0{now.month}"
    now_day = now.day if now.day > 9 else f"0{now.day}"
    now_hour = now.hour if now.hour > 9 else f"0{now.hour}"
    now_minute = now.minute if now.minute > 9 else f"0{now.minute}"
    now_second = now.second if now.second > 9 else f"0{now.second}"

    file_path = f"{now.year}/{now_month}"
    if not os.path.exists(file_path):
        os.makedirs(file_path)
    file_path = f"{file_path}/{now_day}_{now_hour}{now_minute}{now_second}.pdf"
    # file_path = "test.pdf";

    size_societyName = 25
    margin = 10
    doc = SimpleDocTemplate(
        file_path,
        pagesize=A4,
        leftMargin=margin,
        rightMargin=margin,
        topMargin=margin,
        bottomMargin=margin,
    )

    content = []
    styles = getSampleStyleSheet()

    header_table_data = [
        [
            Paragraph(
                f"<font color='red' size={size_societyName}>M</font><font color='black' size={size_societyName}>A</font> SRLS",
                ParagraphStyle(
                    name=any,
                    parent=styles["Normal"],
                    fontName="Times-Roman",  # Cambia con il tuo font preferito
                    fontSize=12,
                ),
            ),
            documentType,
        ],
        ["", ""],
        [
            Paragraph(
                f"Via Cefalonia 24, Brescia (BS)",
                ParagraphStyle(
                    name=any,
                    parent=styles["Normal"],
                    fontName="Times-Roman",
                ),
            ),
            Paragraph(
                f"Spettabile {client_info}",
                ParagraphStyle(
                    name="Centered1",
                    parent=getSampleStyleSheet()["Normal"],
                    fontName="Times-Bold",
                    alignment=1,  # 0=left, 1=center, 2=right
                ),
            ),
        ],
        [
            Paragraph(
                f"P.IVA 04033430986",
                ParagraphStyle(
                    name=any,
                    parent=styles["Normal"],
                    fontName="Times-Roman",
                ),
            ),
            "",
        ],
    ]

    header_table_style = TableStyle(
        [
            # startcol, startrow - endcol, endrow
            ("FONT", (0, 0), (1, 0), "Times-Bold"),
            ("SIZE", (0, 0), (1, 0), 15),
            ("ALIGN", (0, 0), (-1, -1), "CENTER"),
            ("VALIGN", (0, 0), (-1, -1), "MIDDLE"),
            ("TEXTCOLOR", (0, 0), (1, 0), colors.red),
        ]
    )

    header_table = Table(
        header_table_data, colWidths=[doc.width / 2 - 10, doc.width / 2 - 10]
    )
    header_table.setStyle(header_table_style)
    content.append(header_table)

    content.append(Spacer(1, 10))  # Aggiungi spaziatura

    body_table_data = [["Descrizione", "Quantità", "Prezzo €", "Totale € (no IVA)"]]
    for order in orders:
        attributes = [
            order["Description"],
            order["Quantity"],
            order["Price"],
            order["Total"],
        ]
        body_table_data.append(ToParagraph_ForTable(attributes))
    body_table_style = TableStyle(
        [
            ("BACKGROUND", (0, 0), (-1, 0), colors.whitesmoke),
            ("ALIGN", (0, 0), (-1, -1), "CENTER"),
            ("VALIGN", (0, 0), (-1, -1), "MIDDLE"),
            ("GRID", (0, 0), (-1, -1), 1, colors.black),
            ("TEXTCOLOR", (0, 0), (-1, -1), colors.red),
            ("FONTNAME", (0, 0), (-1, 0), "Helvetica-Bold"),
        ]
    )
    body_table = Table(body_table_data, colWidths=[368, 48, 60, 82])
    body_table.setStyle(body_table_style)
    content.append(body_table)

    table_data = [["", ""]]
    styleIva = ParagraphStyle(
        name="Centered1",
        parent=getSampleStyleSheet()["Normal"],
        fontName="Times-Roman",
        alignment=0,  # 0=left, 1=center, 2=right
    )
    styleIvaPrice = ParagraphStyle(
        name="Centered2",
        parent=getSampleStyleSheet()["Normal"],
        fontName="Helvetica",
        alignment=0,  # 0=left, 1=center, 2=right
    )
    noIva = [
        Paragraph(totalObj[0]["text"], styleIva),
        Paragraph(totalObj[0]["value"], styleIvaPrice),
    ]
    iva = [
        Paragraph(totalObj[1]["text"], styleIva),
        Paragraph(totalObj[1]["value"], styleIvaPrice),
    ]
    totale = [
        Paragraph(
            totalObj[2]["text"],
            ParagraphStyle(
                name="Centered1",
                parent=getSampleStyleSheet()["Normal"],
                fontName="Times-Bold",
                alignment=0,  # 0=left, 1=center, 2=right
            ),
        ),
        Paragraph(
            totalObj[2]["value"],
            ParagraphStyle(
                name="Centered2",
                parent=getSampleStyleSheet()["Normal"],
                fontName="Helvetica-Bold",
                alignment=0,  # 0=left, 1=center, 2=right
            ),
        ),
    ]
    table_data.append(noIva)
    table_data.append(iva)
    table_data.append(totale)
    table = Table(table_data, [115, 75])
    table.setStyle(
        TableStyle(
            [
                ("ALIGN", (0, 0), (-1, -1), "LEFT"),  # Align all cells to the left
                ("ALIGN", (0, 0), (-1, 0), "CENTER"),  # Center header text
                ("VALIGN", (0, 0), (-1, -1), "MIDDLE"),  # Center text vertically
                # ("GRID", (0, 0), (-1, -1), 1, colors.black),
            ]
        )
    )
    content.append(table)

    content.append(Spacer(1, 12))  # Aggiungi spaziatura

    for text in finalText:
        content.append(
            Paragraph(
                text,
                ParagraphStyle(
                    name="Centered",
                    parent=getSampleStyleSheet()["Normal"],
                    fontName="Times-Bold",
                    alignment=1,  # 0=left, 1=center, 2=right
                    textColor=colors.red,
                ),
            )
        )

    # Build the PDF
    doc.build(content)

    return file_path
    # try:
    # except Exception as e:
    #    print(f'Error generating PDF: {e}');


def ToParagraph_ForTable(elements: list, setStyle=False):
    style = ParagraphStyle(
        name="Centered1",
        parent=getSampleStyleSheet()["Normal"],
        fontName="Times-Roman",
        alignment=0,  # 0=left, 1=center, 2=right
    )
    if setStyle:
        return [Paragraph(f"{element}", style) for element in elements]
    return [Paragraph(f"{element}") for element in elements]


def DivideAndMergePages(pdf_vecchio, indici_pagine, directorySalvataggio, nomeFile):
    """- DA RISCRIVERE - La funzione usa la libreria PyPDF2 e si occupa di strarre le singole pagine da un dato PDF.
    pdf_vecchio è il reader, generalmente dichiarato come pdfReader = PyPDF2.PdfFileReader(pdfFileObj)
    indici_pagine è la posizione delle pagine specifiche da estrarre; può essere un singolo indice o più di uno; in case fossero tanti, è strettamente necessario sia un array
    directorySalvataggio è il nome della cartella in cui salvare il/i file_estratto
    nomeFile è il nome che verrà assegnato al PDF contenente le pagine divise."""

    file_da_scrivere = pypdf.PdfFileWriter()

    if isinstance(indici_pagine, list):
        for i in indici_pagine:
            file_da_scrivere.addPage(pdf_vecchio.getPage(i))
    else:
        file_da_scrivere.addPage(pdf_vecchio.getPage(indici_pagine))

    # Metto ogni file diviso nell'apposita cartella
    with open(directorySalvataggio + "/" + nomeFile, "wb") as file_estratto:
        file_da_scrivere.write(file_estratto)


def MergePDFs(nome_pdf1, nome_pdf2, directorySalvataggio):
    # Lettura PDF
    # nomePDF = simpledialog.askstring(title=nomeProgramma, prompt="Inserire il nome del file PDF senza l'estensione.\nEsempio: se il file si chiama 'test.pdf', basterà inserire 'test'")
    pdfFileObj_1 = open(directorySalvataggio + "/" + nome_pdf1 + ".pdf", "rb")
    pdfReader_1 = pypdf.PdfFileReader(pdfFileObj_1)
    # print("unisci Ok1" + directorySalvataggio + "/" + nome_pdf2+".pdf")
    pdfFileObj_2 = open(directorySalvataggio + "/" + nome_pdf2 + ".pdf", "rb")
    pdfReader_2 = pypdf.PdfFileReader(pdfFileObj_2)
    # print("unisci Ok2")
    numPagine1 = pdfReader_1.numPages
    numPagine2 = pdfReader_2.numPages

    pdf_unito = pypdf.PdfFileWriter()
    for i in range(numPagine1):
        pdf_unito.addPage(pdfReader_1.getPage(i))
    for i in range(numPagine2):
        pdf_unito.addPage(pdfReader_2.getPage(i))

    with open(directorySalvataggio + "/pdfUnito.pdf", "wb") as file_da_scrivere:
        pdf_unito.write(file_da_scrivere)

    pdfFileObj_1.close()
    pdfFileObj_2.close()
    os.remove(directorySalvataggio + "/" + nome_pdf1 + ".pdf")
    os.remove(directorySalvataggio + "/" + nome_pdf2 + ".pdf")
    os.rename(
        directorySalvataggio + "/pdfUnito.pdf",
        directorySalvataggio + "/" + nome_pdf1 + ".pdf",
    )



import os, shutil;

def CleanPath(path: str, cleanString: str) -> str:
    return path.replace("\\", cleanString).replace('/', cleanString);

def GetSubFolders(folder: str) -> list:
    return [ f.path for f in os.scandir(folder) if f.is_dir() ];

def GetSubFolders(folder: str, level: int) -> list:
    subfolders = [];
    level -= 1;
    if level == 0:
        content = [ f.path for f in os.scandir(folder) if f.is_dir() ];
        if len(content) > 0:
            return content;
        else:
            return [folder];
   
    content = [ f for f in os.scandir(folder) if f.is_dir() ];
    if len(content) > 0:
        for element in content:
            subfolders += GetSubFolders(element.path, level);
    else:
        return [folder];

    return subfolders;

def GetSubFoldersWithFiles_Robocopy(folder: str, level: int) -> list:
    folderForFiles = [];
    level -= 1;
    fileCheckedOnce = False;
    if level == 0:
        for content in os.scandir(folder):
            if content.is_file():
                return [folder];
        return [];
   
    for content in os.scandir(folder):
        if (not fileCheckedOnce) and content.is_file():
            folderForFiles += [folder];
            fileCheckedOnce = True;
        elif content.is_dir():
            folderForFiles += GetSubFoldersWithFiles_Robocopy(content.path, level);

    return folderForFiles;

def GetFolderFiles(folder: str) -> list:
    return [ f.path for f in os.scandir(folder) if f.is_file() ];

def HasFiles(folder: str) -> bool:
    files = os.listdir(folder);
    return len(files) > 0;
   
def GetRobocopyCommand(source: str, elaboration: int = 8, operation:int = 0, onlyFiles=False, customFolder: str = '', checkIfExists: list = [], CalculateDestinationFolder: callable = None) -> list:
    if CalculateDestinationFolder == None:
        return None;
    destination = lambda:CalculateDestinationFolder(source);
    destinationLog = destination.split("/");
    destinationLog = destinationLog[-1];

    logPath = f"{customFolder}{destinationLog.replace(":", "").replace("/", "_") + ("_Files" if onlyFiles else "") + ".log"}";
    i = 0;
    while logPath in checkIfExists:
        i+=1;
        logPath = f"{customFolder}{destinationLog.replace(":", "").replace("/", "_").replace(" ", "_") + ("_Files" if onlyFiles else "") + "_" + str(i) + ".log"}";
    checkIfExists.append(logPath);

    if os.path.isfile(source):
        source = source.split("/");
        if source[-1] == '':
            source = source[:-1];
        fileName = source[-1];
        source = source[:-1];
        source = "/".join(source);

        destination = destination.split("/");
        if destination[-1] == '':
            destination = destination[:-1];
        destination = destination[:-1];
        destination = "/".join(destination);

        command = [
            "Robocopy",
            source,
            destination,
            fileName,
            f"/LOG:{logPath}"
        ];
    else:
        command = [
            "Robocopy",
            source,
            destination,
            f"/LOG:{logPath}"
        ];

        if elaboration in [32, 64, 128]:
            command.append(f"/MT:{elaboration}");
        if not onlyFiles:
            command.append("/E");
        if operation == 1:
            command.append("/MOVE");

    return [command, checkIfExists];

def GetHighestPaths(paths: list) -> list:
    paths.sort();
    highest_paths = [paths[0]];

    for path in paths[1:]:
        if not path.startswith(highest_paths[-1]):
            highest_paths.append(path);

    return highest_paths;

def CreateFolder(folderPath: str):
    pathSplitted = CleanPath(folderPath, "/").split("/");
    if "." in pathSplitted[-1]:
        folderPath = "/".join(pathSplitted[:-1]);
    if not (os.path.exists(folderPath)):
        os.makedirs(folderPath);

def FolderHasContent(folder: str) -> bool:
    if os.path.exists(folder) and os.listdir(folder):
        return True;
    else:
        return False;

def FolderDeleteContents(folder: str) -> list:
    bytes(folder, "utf-8").decode("unicode_escape");
    deleted = [];
    fails = False;
    for filename in os.listdir(folder):
        try:
            file_path = os.path.join(folder, filename);
            if os.path.isfile(file_path) or os.path.islink(file_path):
                os.unlink(file_path);
            elif os.path.isdir(file_path):
                shutil.rmtree(file_path);
            deleted.append(f"Success: {file_path}");
        except Exception as e:
            deleted.append(f"Fail: {e}; Path: {file_path}");
            fails = True;
    return [fails, deleted];


import codicefiscale as CFLib


def PrintBytes(size: int) -> str:
    if size > 1024:
        size = round(size / 1024, 2)
        if size > 1024:
            size = round(size / 1024, 4)
            if size > 1024:
                size = round(size / 1024, 8)
                if size > 1024:
                    size = round(size / 1024, 16)
                    return f"{size} TB"
                else:
                    return f"{size} GB"
            else:
                return f"{size} MB"
        else:
            return f"{size} KB"

    return f"{size} B"


def CheckStringInFile(file_path, target_string):
    try:
        with open(file_path, "r") as file:
            for line in file:
                if target_string in line:
                    return True
        return False
    except FileNotFoundError:
        return False


def CleanArrayFromEmptyString(array: list) -> list:
    return [string for string in array if string != ""]


def ReadFile(file_path: str) -> list:
    with open(file_path, "r") as file:
        lines = file.readlines()
    return lines


def isCodiceFiscale(parola):
    """- DA RIVEDERE - Data in input una stringa, viene analizzata la composizione della stringa per determinare se è un codice fiscale o meno.
    Ritorna True in caso sia un codice fiscale, False altrimenti."""
    trovato = False, parola
    if len(parola) > 16:

        trovato = isCodiceFiscale(parola[0:16])
        if trovato[0]:
            return True, parola[0:16]

        trovato = isCodiceFiscale(parola[-16:])
        if trovato[0]:
            return True, parola[-16:]
        return False, parola

    if len(parola) == 16:
        if (
            (not parola[0:6].isnumeric())
            and (parola[6:8].isnumeric())
            and (not parola[8].isnumeric())
            and (parola[9:11].isnumeric())
            and (not parola[15].isnumeric())
        ):
            return CFLib.IsValid(parola), parola
        return False, parola
    return trovato


def AllElementsHaveSameClass(array: list, cls) -> bool:
    return all(isinstance(element, cls) for element in array)



import pytest;
from AlmaxUtils.Generic import PrintBytes,CheckStringInFile;

def test_PrintBytes():
    assert PrintBytes(500) == "500 B"
    assert PrintBytes(2048) == "2.0 KB"
    assert PrintBytes(2_097_152) == "2.0 MB"
    assert PrintBytes(2_147_483_648) == "2.0 GB"
    assert PrintBytes(2_199_023_255_552) == "2.0 TB"

@pytest.fixture
def temp_file(tmp_path):
    file_path = tmp_path / "temp_file.txt";
    with open(file_path, 'w') as f:
        f.write("Hello, this is a test file.\n")
        f.write("It contains multiple lines.\n")
        f.write("This is the target line.\n")
    return file_path;

def test_CheckStringInFile(temp_file):
    assert CheckStringInFile(temp_file, "target line") == True
    assert CheckStringInFile(temp_file, "not in file") == False
    assert CheckStringInFile("nonexistent_file.txt", "target line") == False


class Payload:
    def __init__(self, data: dict | list | str, log: dict):
        self.__Data = data
        self.__Log = log

    @property
    def First(self) -> str:
        return self.Data[0] if type(self.Data) == list else "Not a list"

    @property
    def Data(self) -> dict | list:
        return self.__Data

class Result:
    def __init__(
        self,
        valid: bool,
        payload: Payload,
        message: str = "Richiesta eseguita con Successo!",
    ):

        self.__IsValid = valid;
        self.__Message = message;
        self.__Payload = payload;

    @property
    def IsValid(self) -> bool:
        return self.__IsValid;

    @property
    def Message(self) -> str:
        return self.__Message;

    @property
    def Payload(self) -> Payload:
        return self.__Payload;



from datetime import datetime, timedelta;

now = datetime.now();

def CalculateTimePassed(start: datetime = 0):
    return datetime.now() - start;

def TimeToString(time: datetime) -> str:
    seconds = time.total_seconds();
    minutes = seconds // 60;
    hours = minutes // 60;
    days = hours // 24;
    return f"{round(days)} Days = {round(hours)} Hours = {round(minutes)} Minutes = {round(seconds, 2)} Seconds";

def GetTimeDesired(AddSeconds: int = 0, Format = "%d-%m-%Y %H:%M:%S") -> str:
    return (datetime.now() + timedelta(seconds=AddSeconds)).strftime(Format);

def LastDayNameOccurence(day_name: str):
    # Dictionary to map day names to their respective numbers
    days = {
        'monday': 0, 'tuesday': 1, 'wednesday': 2, 'thursday': 3,
        'friday': 4, 'saturday': 5, 'sunday': 6
    }
    
    # Get today's date
    today = datetime.today();
    
    # Get the current day of the week (0=Monday, 1=Tuesday, ..., 6=Sunday)
    current_day_num = today.weekday();
    
    # Get the desired day of the week number
    desired_day_num = days[day_name.lower()];
    
    # Calculate the difference in days
    delta_days = (current_day_num - desired_day_num) % 7;
    if delta_days == 0:
        delta_days = 7;
    
    # Calculate the last occurrence date
    last_day_date = today - timedelta(days=delta_days);
    
    return last_day_date;


from __future__ import annotations;

from lxml import etree;
import xml.etree.ElementTree as ET;

from AlmaxUtils import Generic as AUG;
from AlmaxClasses.Result import Result;

class XmlObj:
    '''Classe da utilizzare per estendere una classe che rappresenti un xml con le sue proprietà'''
    def __init__(self):
        pass;

    def GetAttributesForXml(self) -> list:
        attributesValues = [];
        for attr in dir(self):
            if not callable(getattr(self, attr)) and not attr.startswith("_"):
                attributesValues.append(XmlTagAttribute(attr, getattr(self, attr)));
        return attributesValues;
 
    def ConvertToXml(self, parent: ET.Element) -> ET.Element:
        tag = XmlTag(self.__class__.__name__.lower(), self.GetAttributesForXml());
        return tag.ConvertToXmlElement(parent);

class XmlTag:
    '''Classe che facilita la creazione di un elemento tag, da usare in casi un po' più defficili da gestire rispetto a XmlObj'''
    def __init__(self, tagName: str, tagAttributes: list):
        self.__TagName = tagName;
        if AUG.AllElementsHaveSameClass(tagAttributes, XmlTagAttribute):
            self.__TagAttributes = tagAttributes;
        else:
            raise ValueError("All elements in tagAttributes must be instances of XmlTagAttribute");

    @staticmethod
    def FromTouplesList(tagName: str, tagAttributes: list) -> XmlTag:
        allAttr = [];
        for attrName,attrValue in tagAttributes:
            allAttr.append(XmlTagAttribute(attrName, attrValue));
        return XmlTag(tagName, allAttr);

    def ConvertToXmlElement(self, parent: ET.Element) -> ET.Element:
        attribElements = {};
        childs = [];
        for attribute in self.__TagAttributes:
            if isinstance(attribute.Value, list):
                childs.append(attribute.Value);
            else:
                attribElements[attribute.Name] = str(attribute.Value);
        elemForParent = ET.SubElement(parent, self.__TagName, attrib=attribElements);

        for child in childs:
            for elem in child:
                elem.ConvertToXml(elemForParent);

        return elemForParent;

class XmlTagAttribute:
    def __init__(self, attributeName: str, attributeValue: str):
        self.__AttributeName = attributeName;
        self.__AttributeValue = attributeValue;

    @property
    def Name(self) -> str:
        return str(self.__AttributeName);

    @property
    def Value(self):
        return self.__AttributeValue;

def XML_ValidateXSD(xml_path: str, xsd_path: str) -> Result:
    with open(xsd_path, 'r') as schema_file:
        schema = etree.XMLSchema(etree.parse(schema_file));
    with open(xml_path, 'r') as xml_file:
        doc = etree.parse(xml_file);
    try:
        schema.assertValid(doc);
        return Result(True, "XML document is valid");
    except etree.DocumentInvalid as e:
        return Result(False, f"XML document is invalid: {e}");


import pytest;
from datetime import datetime, timedelta;
from unittest.mock import patch;
from AlmaxUtils.Time import CalculateTimePassed, TimeToString, GetTimeDesired;

@pytest.fixture
def fixed_datetime():
    return datetime(2024, 1, 1, 12, 0, 0);

def test_CalculateTimePassed(fixed_datetime):
    start_time = datetime(2024, 1, 1, 10, 0, 0)
    with patch('AlmaxUtils.Time.datetime') as mock_datetime:
        mock_datetime.now.return_value = fixed_datetime
        mock_datetime.side_effect = lambda *args, **kw: datetime(*args, **kw)
        result = CalculateTimePassed(start_time)
        expected = timedelta(hours=2)
        assert result == expected;

def test_TimeToString():
    time = timedelta(days=2, hours=5, minutes=30, seconds=15);
    result = TimeToString(time)
    expected = "2 Days = 53 Hours = 3210 Minutes = 192615.0 Seconds"
    assert result == expected;

def test_AddTime(fixed_datetime):
    seconds_to_add = 5
    with patch('AlmaxUtils.Time.datetime') as mock_datetime:
        mock_datetime.now.return_value = fixed_datetime
        mock_datetime.side_effect = lambda *args, **kw: datetime(*args, **kw)
        result = GetTimeDesired(seconds_to_add, '%d-%m-%Y_%H:%M:%S')
        expected = (fixed_datetime + timedelta(seconds=seconds_to_add)).strftime('%d-%m-%Y_%H:%M:%S')
        assert result == expected;


from setuptools import setup, find_packages;

with open("README.md", "r") as fh:
    readMe = fh.read();

with open("requirements.txt") as f:
    required = f.read().splitlines();

with open("version.txt", "r") as fh:
    versionFile = fh.read().strip();

optional_dependencies = {
    "All": [],
    "Graphics": [
        "tkcalendar==1.6.1",
        "Babel==2.15.0"
    ],
    "PdfManager": [
        "reportlab==4.2.0",
        "pypdf==4.2.0"
    ],
    "XmlManager": [
        "lxml==5.2.2"
    ],
    "AdvancedUtils": [
        "codicefiscale==0.9"
    ]
}

setup(
    name='almax_common',
    version=versionFile,
    description='Library with my most used Classes and Methods',
    long_description=readMe,
    long_description_content_type='text/markdown',
    author='AlMax98',
    author_email='alihaider.maqsood@gmail.com',
    packages=find_packages(),
    package_dir={'': '.'},
    install_requires=[required],
    extra_require=optional_dependencies
);


import os;
from AlmaxUtils.PdfManager import GeneratePdf;
import AlmaxUtils.Time as TimeLib;

class Order:
    def __init__(self, description, quantity, price):
        self.Description = description;
        self.Quantity = "{:,.1f}".format(quantity);
        self.Price = "{:,.2f}".format(price);
        self.Total = "{:,.2f}".format(quantity * price)

    def ToDict(self):
        return vars(self)

def test_GeneratePdf():
    now = TimeLib.now;
    now_month = now.month if now.month > 9 else f"0{now.month}";
    now_day = now.day if now.day > 9 else f"0{now.day}";
    now_hour = now.hour if now.hour > 9 else f"0{now.hour}";
    now_minute = now.minute if now.minute > 9 else f"0{now.minute}";
    now_second = now.second if now.second > 9 else f"0{now.second}";
    file_path = f"{now.year}/{now_month}/{now_day}_{now_hour}{now_minute}{now_second}.pdf";

    client_info = {
        "name": "Ali Srls"
    };
    orders = [
        Order('fbewkjfbwebfjewbfkjewbfkewfewfvjewbfjwebfjkewjfbewfjwebjfwejfjwebfjewbfjwefwebfwefweeffwe', 2.6, 31.5),
        Order('ciao', 2.6, 34090.5),
        Order('fbewkjfbwebfjewbfkjewbfkewfewfvjewbfjwebfjkewjfbewfjwebjfwejfjwebfjewbfjwefwebfwefweeffwe', 2.6, 32.5),
        Order('fbewkjfbwebfjewbfkjewbfkewfewfvjewbfjwebfjkewjfbewfjwebjfwejfjwebfjewbfjwefwebfwefweeffwe', 2.6, 33.5),
        Order('fbewkjfbwebfjewbfkjewbfkewfewfvjewbfjwebfjkewjfbewfjwebjfwejfjwebfjewbfjwefwebfwefweeffwe', 2.6, 34.5)
    ];

    GeneratePdf(
        client_info,
        [order.ToDict() for order in orders],
        [
            {"text": "TOTALE SENZA IVA", "value": f"{10}€"},
            {"text": "IVA al 22%", "value": f"{100}€"},
            {"text": "TOTALE CON IVA", "value": f"{20}€"},
        ],
        [f"NB: Qualsiasi modifica che non è citata, sarà pagata a parte."],
        "Test"
    )
    assert os.path.exists(file_path)
